import {
  select,
  selectAll,
  csv,
  scaleLinear,
  extent,
  axisLeft,
  axisBottom,
  format,
  scaleOrdinal,
} from 'd3';

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

const render = data => {
  
  //const title = 'Graduate programs: employment% vs median salary';
  const title = 'Graduate programs: total graduates vs median salary';
  
  //const xValue = d => d.Employment_Rate;
  //const xAxisLabel = 'Employment Rate';
  const xValue = d => d.Grad_total;
  const xAxisLabel = 'Total graduates';
  
  const yValue = d => d.Grad_median;
  const circleRadius = 6;
  const yAxisLabel = 'Median Salary';
  
  // create color value
  const colorScheme = [
    '#1b69a1', '#aec7e8',
    '#ff7f0e', 'yellow',
    '#2ca02c', '#98df8a',
    '#d62728', '#ff9896',
    '#9467bd', '#c5b0d5',
    '#8c564b', '#c49c94',
    '#e377c2', '#f7b6d2',
    '#7f7f7f', '#c7c7c7',
    '#bcbd22', '#dbdb8d',
    '#17becf', '#9edae5'];
  
  const category = [...new Set(data.map(d => d.Major_category))];
  //console.log(category)
  const colorValue = d => d.Major_category;
  const colorLabel = 'Major category';
  const colorScale = scaleOrdinal()
    .domain(category)
    .range(colorScheme);
  
  const margin = { top: 60, right: 270, bottom: 75, left: 100 };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();
  
  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
    .nice();
  
  const g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);
  
  const AxisTickFormat = number =>
    format('.2s')(number);
  
  const xAxis = axisBottom(xScale)
    .tickFormat(AxisTickFormat)
    .tickSize(-innerHeight)
    .tickPadding(15);
  
  const yAxis = axisLeft(yScale)
    .tickFormat(AxisTickFormat)
    .tickSize(-innerWidth)
    .tickPadding(10);
  
  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();
  
  yAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', -60)
      .attr('x', -innerHeight / 2)
      .attr('fill', 'black')
      .attr('transform', `rotate(-90)`)
      .attr('text-anchor', 'middle')
      .text(yAxisLabel);
  
  const xAxisG = g.append('g').call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);
  
  xAxisG.select('.domain').remove();
  
  xAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', 65)
      .attr('x', innerWidth / 2)
      .attr('fill', 'black')
      .text(xAxisLabel);
  
  // create legend group
  const colorLegendG = g.selectAll('.legend')
    .data(colorScale.domain())
    .enter().append('g')
      .attr('class', 'legend')
      .attr('transform', `translate(600, 20)`);
  
  // create legend circles
  colorLegendG.append('circle')
    .attr('class', 'legend-circle')
    .attr('r', 6)
    .attr('fill', colorScale)
    .attr('transform', (d, i) =>
        `translate(0, ${i * 22})`);

  // create legend text
  colorLegendG.append('text')
    .attr('class', 'legend-text')
    .attr('transform', (d, i) =>
        `translate(0, ${i * 22})`
    )
    .attr('x', circleRadius * 3)
    .attr('dy', '0.32em')
    .text(d => d);
  
  
  // create the click event
  let selected;
  colorLegendG.on('click', (type, i, nodes) => {
    
    selectAll('.data-circle').attr('opacity', 1);
	selectAll('.legend').attr('opacity', 1);
    
    if (selected !== type) {
      selectAll('.legend').attr('opacity', 0.1);
      select(nodes[i])
        .attr('opacity', 1);
      selectAll('.data-circle')
        .filter(d => {
          return d.Major_category !== type;})
          .attr('opacity', 0.1);
      selected = type;
    }
    else {selected = null;}
  });
  
  g.selectAll('circle').data(data)
    .enter().append('circle')
      .attr('class', 'data-circle')
      .attr('cy', d => yScale(yValue(d)))
      .attr('cx', d => xScale(xValue(d)))
      .attr('r', circleRadius)
      // color differentiate dots by category
      .attr('fill', d => colorScale(colorValue(d)));
  
  g.append('text')
      .attr('class', 'title')
      .attr('y', -10)
      .text(title);
};

csv('https://vizhub.com/ilovemanu/datasets/gradudate_students.csv')
  .then(data => {
    data.forEach(d => {
      d.Major_category = d.Major_category;
      d.Major = d.Major;
      d.Grad_total = +d.Grad_total;
      d.Grad_employed = +d.Grad_employed;
      d.Grad_unemployed = +d.Grad_unemployed;
      d.Grad_unemployment_rate = +d.Grad_unemployment_rate;
      d.Grad_median = +d.Grad_median;
      d.Employment_Rate = +(+d.Grad_employed)/(+d.Grad_total);
    });
    render(data);
  });